#include "stuff.h"
#include "jspfile.h"
#define WINGDI_FUDGE
#include <windows.h>

extern int recolortool(jspfile& file, int& sprite);

void jspedit(string filename) {
    jspfile file;
    jspfile::load(file, filename);

    // static stuff
    const int bgsize = 40, crosssize = 3;

    Bitmap background(SCREEN_W, SCREEN_H);
    background.clear(PAL_GRAY + 16);
    for (int x = 0; x < SCREEN_W; x += bgsize) {
        for (int y = 0; y < SCREEN_H; y += bgsize) {
            if ((x/bgsize + y/bgsize) % 2 == 0) background.fillrect(x, y, x + bgsize - 1, y + bgsize - 1, PAL_GRAY + 24);
        }
    }
    background.fillrect(0, 0, SCREEN_W - 1, 3*bgsize - 1, PAL_GRAY + 31);
    background.line(0, 3*bgsize - 1, SCREEN_W - 1, 3*bgsize - 1, PAL_GRAY);

    background.textout(font, "JSP Archive Utility - by SpaceManiac", 3, 3, PAL_GRAY);
    background.textout(font, "Type to change filename, Ctrl-V to paste", 3, 14, PAL_GRAY);
    background.textout(font, "Ctrl-S to save JSP, Ctrl-L to load JSP", 3, 36, PAL_GRAY);
    background.textout(font, "Ctrl-I to import BMP, Ctrl-E to export BMP", 3, 47, PAL_GRAY);
    background.textout(font, "(In BMPs, index 0/black is transparent)", 3, 58, PAL_GRAY);
    background.textout(font, "L/R to navigate, Ctrl-L/R to shift sprite", 3, 69, PAL_GRAY);
    background.textout(font, "Ctrl-N to add blank, Ctrl-X to remove sprite", 3, 80, PAL_GRAY);
    background.textout(font, "Click and drag to change sprite origin", 3, 91, PAL_GRAY);
    background.textout(font, "Hit Ctrl-R to bring up recolor tool", 3, 103, PAL_GRAY);

    // main loop

    Bitmap buffer(SCREEN_W, SCREEN_H);
    int sprite = 0;
    int timer = 0;
    int lastx = 0, lasty = 0;
    bool lastpress = false;
    string status = "Startup successful";

    int done = RET_NONE;
    while (done == RET_NONE) {
        if (close_button_pressed) done = RET_OK;
        if (keypressed()) {
            int k = readkey();
            char ch = k & 0xff;
            int scan = k >> 8;
            bool ctrl = key[KEY_LCONTROL] || key[KEY_RCONTROL];

            if (scan == KEY_ESC) {
                done = RET_OK;
            } else if (scan == KEY_LEFT) {
                if (sprite > 0) {
                    if (ctrl) {
                        swap(file.images[sprite - 1], file.images[sprite]);
                        swap(file.imageInfo[sprite - 1], file.imageInfo[sprite]);
                        status = "Sprite shifted";
                    }
                    --sprite;
                }
            } else if (scan == KEY_RIGHT) {
                if (sprite < file.imageCount - 1) {
                    if (ctrl) {
                        swap(file.images[sprite], file.images[sprite + 1]);
                        swap(file.imageInfo[sprite], file.imageInfo[sprite + 1]);
                        status = "Sprite shifted";
                    }
                    ++sprite;
                }
            } else if (ctrl) {
                ch += 'a' - 1;
                if (ch == 's') {
                    if (jspfile::save(file, filename)) {
                        status = "Save successful";
                    } else {
                        status = "Save failed";
                    }
                } else if (ch == 'l') {
                    if (jspfile::load(file, filename)) {
                        sprite = 0;
                        status = "Load successful";
                    } else {
                        status = "Load failed";
                    }
                } else if (ch == 'i') {
                    Bitmap temp;
                    if (temp.load(filename)) {
                        file.images[sprite] = temp;
                        file.imageInfo[sprite].width = temp->w;
                        file.imageInfo[sprite].height = temp->h;
                        file.imageInfo[sprite].byteSize = 0;
                        file.imageInfo[sprite].mystery[0] = 0;
                        file.imageInfo[sprite].mystery[1] = 0;
                        status = "Import successful";
                    } else {
                        status = "Import failed";
                    }
                } else if (ch == 'e') {
                    PALETTE palette;
                    get_palette(palette);
                    if (save_bitmap(filename.c_str(), file.images[sprite].ptr(), palette) == 0) {
                        status = "Export successful";
                    } else {
                        status = "Export failed";
                    }
                } else if (ch == 'n') {
                    sprite = file.imageCount;
                    jspfile::info info = { 0, 0, 0, 0, 0, { 0, 0 } };
                    file.images.push_back(Bitmap(0, 0));
                    file.imageInfo.push_back(info);
                    file.imageCount++;
                    status = "New sprite added";
                } else if (ch == 'x') {
                    file.images.erase(file.images.begin() + sprite);
                    file.imageInfo.erase(file.imageInfo.begin() + sprite);
                    file.imageCount--;
                    if (sprite >= file.imageCount) sprite = file.imageCount - 1;
                    status = "Sprite removed";
                } else if (ch == 'r') {
                    if (file.imageCount > 0) {
                        int i = recolortool(file, sprite);
                        if (i != RET_OK) done = RET_QUIT;
                        status = "Recolor tool exited";
                    } else {
                        status = "No images to recolor";
                    }
                } else if (ch == 'v') {
                    string text;
                    if (OpenClipboard(NULL)) {
                        text = (char*) GetClipboardData(CF_TEXT);
                        CloseClipboard();
                    }
                    if (text != "") filename = text;
                }
            } else if (scan == KEY_BACKSPACE || scan == KEY_DEL) {
                if (filename.length() > 0) filename = filename.substr(0, filename.length() - 1);
                status = "";
            } else if (ch == ' ' || isprint(ch)) {
                filename += ch;
                status = "";
            }
        }

        bool press = (mouse_b & 1);
        if (press && file.imageCount > 0) {
            if (lastpress) {
                int dx = mouse_x - lastx, dy = mouse_y - lasty;
                file.imageInfo[sprite].originX -= dx;
                file.imageInfo[sprite].originY -= dy;
                lastx += dx; lasty += dy;
            } else {
                lastx = mouse_x; lasty = mouse_y;
                lastpress = true;
            }
        } else {
            lastpress = false;
        }

        buffer.draw(background, 0, 0);
        buffer.textout(font, status, 403, 13, PAL_GRAY);
        buffer.textout(font, "Filename: " + filename + (++timer % 10 < 5 ? "_" : ""), 3, 25, PAL_GRAY);
        if (file.imageCount == 0) {
            buffer.textout(font, "No sprites loaded", 403, 36, PAL_GRAY);
        } else {
            jspfile::info i = file.imageInfo[sprite];
            buffer.textout(font, "Sprite " + toString(sprite + 1) + " of " + toString(file.imageCount), 403, 36, PAL_GRAY);
            buffer.textout(font, "Dimensions: " + toString(i.width) + " x " + toString(i.height), 403, 47, PAL_GRAY);
            buffer.textout(font, "Origin: (" + toString(i.originX) + ", " + toString(i.originY) + ")", 403, 58, PAL_GRAY);
            buffer.textout(font, "Mystery 1: " + toString(i.mystery[0]), 403, 69, PAL_GRAY);
            buffer.textout(font, "Mystery 2: " + toString(i.mystery[1]), 403, 80, PAL_GRAY);
            buffer.textout(font, "Compressed size: " + (i.byteSize != 0 ? toString(i.byteSize) : string("Unknown")), 403, 91, PAL_GRAY);
            buffer.draw_sprite(file.images[sprite], 400 - i.originX, 360 - i.originY);
            buffer.line(400 - crosssize, 360, 400 + crosssize, 360, PAL_GRAY);
            buffer.line(400, 360 - crosssize, 400, 360 + crosssize, PAL_GRAY);
        }
        buffer.draw_to_screen();
    }
}
